@@ -21,6 +21,7 @@
 #include <utility>
 #include <cstdint>
 #include <algorithm>
+#include <functional> // for std::invoke
 
 #include "sycl_defs.h"
 #include "parallel_backend_sycl_utils.h"
@@ -183,7 +184,7 @@ __radix_sort_count_submit(sycl::queue& __q, std::size_t __segments, std::size_t
     constexpr ::std::uint32_t __radix_states = 1 << __radix_bits;
 
     // iteration space info
-    const ::std::size_t __n = __val_rng.size();
+    const ::std::size_t __n = oneapi::dpl::__ranges::__size(__val_rng);
     const ::std::size_t __elem_per_segment = oneapi::dpl::__internal::__dpl_ceiling_div(__n, __segments);
     const ::std::size_t __no_op_flag_idx = __count_buf.size() - 1;
 
@@ -219,7 +220,7 @@ __radix_sort_count_submit(sycl::queue& __q, std::size_t __segments, std::size_t
                      __val_idx += __wg_size)
                 {
                     // get the bucket for the bit-ordered input value, applying the offset and mask for radix bits
-                    auto __val = __order_preserving_cast<__is_ascending>(__proj(__val_rng[__val_idx]));
+                    auto __val = __order_preserving_cast<__is_ascending>(std::invoke(__proj, __val_rng[__val_idx]));
                     ::std::uint32_t __bucket = __get_bucket<(1 << __radix_bits) - 1>(__val, __radix_offset);
                     // increment counter for this bit bucket
                     ++__count_arr[__bucket];
@@ -472,14 +473,14 @@ struct __peer_prefix_helper<__radix_states, _OffsetT, __peer_prefix_algo::subgro
 
 template <typename _InRange, typename _OutRange>
 void
-__copy_kernel_for_radix_sort(::std::size_t __segments, const ::std::size_t __elem_per_segment, ::std::size_t __sg_size,
-                         sycl::nd_item<1> __self_item, _InRange& __input_rng, _OutRange& __output_rng)
+__copy_kernel_for_radix_sort(const std::size_t __elem_per_segment, std::size_t __sg_size, sycl::nd_item<1> __self_item,
+                             _InRange& __input_rng, _OutRange& __output_rng)
 {
     // item info
     const ::std::size_t __self_lidx = __self_item.get_local_id(0);
     const ::std::size_t __wgroup_idx = __self_item.get_group(0);
     const ::std::size_t __seg_start = __elem_per_segment * __wgroup_idx;
-    const ::std::size_t __n = __output_rng.size();
+    const ::std::size_t __n = oneapi::dpl::__ranges::__size(__output_rng);
 
     ::std::size_t __seg_end = sycl::min(__seg_start + __elem_per_segment, __n);
     // ensure that each work item in a subgroup does the same number of loop iterations
@@ -522,10 +523,10 @@ __radix_sort_reorder_submit(sycl::queue& __q, std::size_t __segments, std::size_
     using _ValueT = oneapi::dpl::__internal::__value_t<_InRange>;
     using _PeerHelper = __peer_prefix_helper<__radix_states, _OffsetT, _PeerAlgo>;
 
-    assert(__input_rng.size() == __output_rng.size());
+    assert(oneapi::dpl::__ranges::__size(__input_rng) == oneapi::dpl::__ranges::__size(__output_rng));
 
     // iteration space info
-    const ::std::size_t __n = __output_rng.size();
+    const ::std::size_t __n = oneapi::dpl::__ranges::__size(__output_rng);
     const ::std::size_t __elem_per_segment = oneapi::dpl::__internal::__dpl_ceiling_div(__n, __segments);
 
     const ::std::size_t __no_op_flag_idx = __offset_buf.size() - 1;
@@ -557,8 +558,7 @@ __radix_sort_reorder_submit(sycl::queue& __q, std::size_t __segments, std::size_
                 auto& __no_op_flag = __offset_rng[__no_op_flag_idx];
                 if (__no_op_flag)
                 {
-                    __copy_kernel_for_radix_sort(__segments, __elem_per_segment, __sg_size, __self_item, __input_rng,
-                                                 __output_rng);
+                    __copy_kernel_for_radix_sort(__elem_per_segment, __sg_size, __self_item, __input_rng, __output_rng);
                     return;
                 }
 
@@ -598,7 +598,7 @@ __radix_sort_reorder_submit(sycl::queue& __q, std::size_t __segments, std::size_
                     _ValueT __in_val = std::move(__input_rng[__val_idx]);
                     // get the bucket for the bit-ordered input value, applying the offset and mask for radix bits
                     ::std::uint32_t __bucket = __get_bucket<(1 << __radix_bits) - 1>(
-                        __order_preserving_cast<__is_ascending>(__proj(__in_val)), __radix_offset);
+                        __order_preserving_cast<__is_ascending>(std::invoke(__proj, __in_val)), __radix_offset);
 
                     const auto __new_offset_idx = __peer_prefix_hlp.__peer_contribution(__bucket, __offset_arr);
                     __output_rng[__new_offset_idx] = std::move(__in_val);
@@ -615,7 +615,7 @@ __radix_sort_reorder_submit(sycl::queue& __q, std::size_t __segments, std::size_
                         new (&__in_val.__v) _ValueT(std::move(__input_rng[__seg_end + __self_lidx]));
 
                         __bucket = __get_bucket<(1 << __radix_bits) - 1>(
-                            __order_preserving_cast<__is_ascending>(__proj(__in_val.__v)), __radix_offset);
+                            __order_preserving_cast<__is_ascending>(std::invoke(__proj, __in_val.__v)), __radix_offset);
                     }
                     const auto __new_offset_idx = __peer_prefix_hlp.__peer_contribution(__bucket, __offset_arr);
                     if (__self_lidx < __residual)
@@ -669,7 +669,7 @@ struct __parallel_radix_sort_iteration
         // This value exceeds the current practical limit for GPUs, but may need to be re-evaluated in the future.
         std::size_t __scan_wg_size = oneapi::dpl::__internal::__max_work_group_size(__q, (std::size_t)4096);
 #if _ONEDPL_RADIX_WORKLOAD_TUNING
-        ::std::size_t __count_wg_size = (__in_rng.size() > (1 << 21) /*2M*/ ? 128 : __max_sg_size);
+        ::std::size_t __count_wg_size = (oneapi::dpl::__ranges::__size(__in_rng) > (1 << 21) /*2M*/ ? 128 : __max_sg_size);
 #else
         ::std::size_t __count_wg_size = __max_sg_size;
 #endif
@@ -687,6 +687,22 @@ struct __parallel_radix_sort_iteration
         __scan_wg_size =
             sycl::min(__scan_wg_size, oneapi::dpl::__internal::__kernel_work_group_size(__q, __local_scan_kernel));
         __count_wg_size = sycl::max(__count_sg_size, __reorder_sg_size);
+#else
+        // When kernel compilation is disabled, use conservative fallback values
+        // Get device sub-group sizes and pick a suitable one for radix sort
+        const auto __subgroup_sizes = __q.get_device().template get_info<sycl::info::device::sub_group_sizes>();
+        // The radix sort kernels are optimized for sub-group size 16 to avoid register spills
+        // and efficiently handle 4-bit radix (16 buckets). Prefer 16, then 32, then 8.
+        if (std::find(__subgroup_sizes.begin(), __subgroup_sizes.end(), 16) != __subgroup_sizes.end())
+            __reorder_sg_size = 16;
+        else if (std::find(__subgroup_sizes.begin(), __subgroup_sizes.end(), 32) != __subgroup_sizes.end())
+            __reorder_sg_size = 32;
+        else if (std::find(__subgroup_sizes.begin(), __subgroup_sizes.end(), 8) != __subgroup_sizes.end())
+            __reorder_sg_size = 8;
+        // else keep __reorder_sg_size = __max_sg_size
+
+        // For __count_wg_size, use the maximum of the current value and __reorder_sg_size
+        __count_wg_size = sycl::max(__count_wg_size, __reorder_sg_size);
 #endif
         const ::std::uint32_t __radix_states = 1 << __radix_bits;
 
@@ -714,7 +730,7 @@ struct __parallel_radix_sort_iteration
 
         // 2. Scan Phase
         sycl::event __scan_event = __radix_sort_scan_submit<_RadixLocalScanKernel, __radix_bits>(
-            __q, __scan_wg_size, __segments, __tmp_buf, __in_rng.size(), __count_event
+            __q, __scan_wg_size, __segments, __tmp_buf, oneapi::dpl::__ranges::__size(__in_rng), __count_event
 #if _ONEDPL_COMPILE_KERNEL
             , __local_scan_kernel
 #endif
@@ -768,7 +784,7 @@ __future<sycl::event>
 __parallel_radix_sort(oneapi::dpl::__internal::__device_backend_tag, _ExecutionPolicy&& __exec, _Range&& __in_rng,
                       _Proj __proj)
 {
-    const ::std::size_t __n = __in_rng.size();
+    const ::std::size_t __n = oneapi::dpl::__ranges::__size(__in_rng);
     assert(__n > 1);
 
     // types
