@@ -80,12 +80,8 @@
     (!_ONEDPL_LIBSYCL_VERSION_LESS_THAN(50700) || __SYCL_COMPILER_VERSION > 20201214)
 #define _ONEDPL_SYCL2020_BUFFER_ALLOCATOR_PRESENT             (!_ONEDPL_LIBSYCL_VERSION_LESS_THAN(60000))
 #define _ONEDPL_SYCL2020_LOCAL_ACCESSOR_PRESENT               (!_ONEDPL_LIBSYCL_VERSION_LESS_THAN(60000))
-// The unified future supporting USM memory and buffers is only supported after DPC++ 2023.1 but not by 2023.2.
-#define _ONEDPL_SYCL2020_DEFAULT_ACCESSOR_CONSTRUCTOR_PRESENT                                                          \
-    (!_ONEDPL_LIBSYCL_VERSION_LESS_THAN(60100) && _ONEDPL_LIBSYCL_VERSION != 60200)
 #define _ONEDPL_SYCL2020_HOST_TARGET_PRESENT                  (!_ONEDPL_LIBSYCL_VERSION_LESS_THAN(60200))
 #define _ONEDPL_SYCL2020_HOST_ACCESSOR_PRESENT                (!_ONEDPL_LIBSYCL_VERSION_LESS_THAN(60200))
-#define _ONEDPL_SYCL2020_GET_HOST_ACCESS_PRESENT              (!_ONEDPL_LIBSYCL_VERSION_LESS_THAN(60200))
 #define _ONEDPL_SYCL2020_LOCAL_ACC_GET_MULTI_PTR_PRESENT      (!_ONEDPL_LIBSYCL_VERSION_LESS_THAN(70000))
 
 // Feature macros for DPC++ SYCL runtime library alternatives to non-supported SYCL 2020 features
@@ -108,6 +104,9 @@
 #endif
 
 #define _ONEDPL_SYCL_DEVICE_COPYABLE_SPECIALIZATION_BROKEN (_ONEDPL_LIBSYCL_VERSION_LESS_THAN(70100))
+// There are issues with some accessor constructors prior to DPC++ 2023.1 and with 2023.2.
+#define _ONEDPL_SYCL2020_DEFAULT_ACCESSOR_CONSTRUCTOR_BROKEN                                                           \
+    (_ONEDPL_LIBSYCL_VERSION_LESS_THAN(60100) || _ONEDPL_LIBSYCL_VERSION == 60200)
 
 // Macro to check if we are compiling for SPIR-V devices. This macro must only be used within
 // SYCL kernels for determining SPIR-V compilation. Using this macro on the host may lead to incorrect behavior.
@@ -528,19 +527,6 @@ using __local_accessor =
 #    error "sycl::local_accessor is not supported, and no alternative is available"
 #endif
 
-template <typename _Buf>
-auto
-__get_host_access(_Buf&& __buf)
-{
-#if _ONEDPL_SYCL2020_GET_HOST_ACCESS_PRESENT
-    return ::std::forward<_Buf>(__buf).get_host_access(sycl::read_only);
-#elif _ONEDPL_LIBSYCL_VERSION_LESS_THAN(60200)
-    return ::std::forward<_Buf>(__buf).template get_access<sycl::access::mode::read>();
-#else
-#    error "sycl::buffer::get_host_access is not supported, and no alternative is available"
-#endif
-}
-
 template <typename _Acc>
 auto
 __get_accessor_ptr(const _Acc& __acc)
